#!/bin/sh -v
decrypt test <foo.02 >bar.02
decrypt test <foo.03 >bar.03
decrypt test <foo.04 >bar.04
decrypt test <foo.05 >bar.05
decrypt test <foo.06 >bar.06
decrypt test <foo.07 >bar.07
decrypt test <foo.08 >bar.08
decrypt test <foo.09 >bar.09
decrypt test <foo.10 >bar.10
decrypt test <foo.11 >bar.11
decrypt test <foo.12 >bar.12
decrypt test <foo.13 >bar.13
decrypt test <foo.14 >bar.14
decrypt test <foo.15 >bar.15
